"""
2021.5.20 create by kefeng.zheng
默认请求对象
"""
import json

import requests

from basepay_sdk.log_util import log_error, log_info
from basepay_sdk.rsa_utils import sha256_rsa_sign, sha256_rsa_design
from basepay_sdk.basepay import BasePay

class ApiRequest:

	@staticmethod
	def get_request_url(func_code):
		base_url = BasePay.base_url
		if not BasePay.prod_mode:
			base_url = BasePay.base_url_mock

		if (not base_url.endswith("/")):
			base_url = base_url + "/"

		interface_url = func_code.replace(".", "/")
		request_url = base_url + interface_url
		return request_url

	@staticmethod
	def post(func_code, request_params, merchant_key = 'default', request_file=None):
		return ApiRequest._request(func_code, request_params, 'post', merchant_key, request_file)

	@staticmethod
	def get(func_code, request_params, merchant_key = 'default'):
		return ApiRequest._request(func_code, request_params, 'get', merchant_key)

	@staticmethod
	def _build_request_info(mer_config, params):
		"""
		根据请求方式构造请求头和请求参数
		:return: header 请求头 params 请求参数
		"""
		if (not mer_config.get('product_id')):
			log_error('未配置产品号')
			raise RuntimeError('未配置产品号')
		product_id = mer_config.get('product_id')
		# 构造请求头
		header = {
		  'product_id': product_id,
		  "format": 'JSON',
		  'charset': 'UTF-8',
		  'version': '1.0.0',
		  'sdk_version': BasePay.sdk_version
		}

		# 获取商户密钥
		if not mer_config.get('rsa_private_key'):
			log_error('未配置私钥')
			raise RuntimeError('未配置私钥')
			
		data = json.dumps(params);
		
		# 对请求参数进行加签
		flag, cipher_text = sha256_rsa_sign(mer_config.get('rsa_private_key'), data, 'utf-8')

		if not flag:
			log_error('签名错误 签名原文{} '.format(data))
			raise RuntimeError('报文签名失败')
			
		if not mer_config.get('sys_id'):
			log_error('未配置系统号')
			raise RuntimeError('未配置系统号')

		# 组装上送参数
		post_params = {
			'data': data,
			'sign': cipher_text,
			'sign_type': 'RSA2',
			'sys_id': mer_config['sys_id'] #请求系统号，由汇付分配
		}

		log_info('request param is {}, \nhead is {}'.format(post_params, header))

		return header, post_params

	@staticmethod
	def _request(func_code, request_params, method, merchant_key, files=None):
		"""
		执行请求
		:func_code: 接口功能编码
		:param method: 请求方法类型
		:param files: 上传的文件
		:return: 网路请求返回的数据
		"""
		request_url = ApiRequest.get_request_url(func_code)
		
		# 获取商户配置信息
		if (not BasePay.mer_configs.get(merchant_key)):
			log_error('指定的商户配置信息不存在，merchant_key={}'.format(merchant_key))
			raise RuntimeError('指定的商户配置信息不存在')
		mer_config = BasePay.mer_configs.get(merchant_key)

		header, params = ApiRequest._build_request_info(mer_config, request_params)

		http_method = getattr(requests, method or 'post')

		if files:
			resp = http_method(request_url, data=params, files=files, timeout=30,
							   headers=header)
		elif method == 'post':
			resp = http_method(request_url, json=params, files=files, timeout=30,
							   headers=header)
		else:
			resp = http_method(request_url, params, timeout=30, headers=header)

		log_info('request to {}, resp is {}'.format(request_url, resp.text))

		return ApiRequest._build_return_data(resp, mer_config)

	@staticmethod
	def _build_return_data(resp, mer_config):
		# 将返回报文转换为json
		try:
			resp_json = json.loads(resp.text)
		except Exception as e:
			log_error('basepay resp_code is ' + str(resp.status_code))
			log_error(str(e))
			return resp.text
			
		log_error(resp_json);
			
		# 服务端返回数据
		resp_code = resp_json.get('resp_code', '')
		resp_desc = resp_json.get('resp_desc', '')
		if not resp_code == '10000':
			log_error('接口调用失败处理' + resp_desc)
			return resp_json

		# 服务端返回数据
		data = resp_json.get('data', '')
		# 返回字段中的签名
		resp_sign = resp_json.get('sign', '')

		if not resp_sign:
			# 如果没有签名字段，直接返回内容，一般为404等
			return resp_json

		# 当业务请求成功时验证返回数据与签名
		if not mer_config.get('rsa_public_key'):
			log_error('未配置验签公钥')
			raise RuntimeError('未配置验签公钥')

		# 验证返回数据与返回加签结果是否一致
		flag, info = sha256_rsa_design(resp_sign, data, mer_config['rsa_public_key'])

		if not flag:
			# 如果验签失败，抛出异常
			log_error('验签失败!{}'.format(info))
			raise RuntimeError(info)

		return json.loads(data)
